<?php

namespace Litecms\Promo\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Repository\Filter\RequestFilter;
use Litecms\Promo\Forms\Promo as PromoForm;
use Litecms\Promo\Http\Requests\PromoRequest;
use Litecms\Promo\Interfaces\PromoRepositoryInterface;
use Litecms\Promo\Repositories\Eloquent\Filters\PromoResourceFilter;
use Litecms\Promo\Repositories\Eloquent\Presenters\PromoListPresenter;

/**
 * Resource controller class for promo.
 */
class PromoResourceController extends BaseController
{

    /**
     * Initialize promo resource controller.
     *
     *
     * @return null
     */
    public function __construct(PromoRepositoryInterface $promo)
    {
        parent::__construct();
        $this->form = PromoForm::setAttributes()->toArray();
        $this->modules = $this->modules(config('litecms.promo.modules'), 'promo', guard_url('promo'));
        $this->repository = $promo;
    }

    /**
     * Display a list of promo.
     *
     * @return Response
     */
    public function index(PromoRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $data = $this->repository
            ->pushFilter(RequestFilter::class)
            ->pushFilter(PromoResourceFilter::class)
            ->setPresenter(PromoListPresenter::class)
            ->simplePaginate($pageLimit)
            // ->withQueryString()
            ->toArray();

        extract($data);
        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('promo::promo.names'))
            ->view('promo::promo.index')
            ->data(compact('data', 'meta', 'links', 'modules', 'form'))
            ->output();
    }

    /**
     * Display promo.
     *
     * @param Request $request
     * @param Model   $promo
     *
     * @return Response
     */
    public function show(PromoRequest $request, PromoRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('promo::promo.name'))
            ->data(compact('data', 'form', 'modules', 'form'))
            ->view('promo::promo.show')
            ->output();
    }

    /**
     * Show the form for creating a new promo.
     *
     * @param Request $request
     *x
     * @return Response
     */
    public function create(PromoRequest $request, PromoRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('promo::promo.name'))
            ->view('promo::promo.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Create new promo.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(PromoRequest $request, PromoRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $repository->create($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.created', ['Module' => trans('promo::promo.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('promo/promo/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/promo/promo'))
                ->redirect();
        }

    }

    /**
     * Show promo for editing.
     *
     * @param Request $request
     * @param Model   $promo
     *
     * @return Response
     */
    public function edit(PromoRequest $request, PromoRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('promo::promo.name'))
            ->view('promo::promo.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Update the promo.
     *
     * @param Request $request
     * @param Model   $promo
     *
     * @return Response
     */
    public function update(PromoRequest $request, PromoRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $repository->update($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('promo::promo.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('promo/promo/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('promo/promo/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the promo.
     *
     * @param Model   $promo
     *
     * @return Response
     */
    public function destroy(PromoRequest $request, PromoRepositoryInterface $repository)
    {
        try {
            $repository->delete();
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('promo::promo.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('promo/promo/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('promo/promo/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }
}
